package dal

import (
	"context"
	"gitea.pena/PenaSide/penadisk/dal/yaCloud"
	"github.com/minio/minio-go/v7"
)

func (mc *MinioClient) IsBucketExists(ctx context.Context) (bool, error) {
	ok, err := mc.client.BucketExists(ctx, mc.BucketName)
	if mc.err(err) {
		return false, err
	}

	return ok, nil
}

func (mc *MinioClient) CreateBucket(ctx context.Context) error {
	err := mc.client.MakeBucket(ctx, mc.BucketName, minio.MakeBucketOptions{ObjectLocking: false})

	if mc.err(err) {
		return err
	}

	return err
}

// GetBucketInfo - получить данные по корзине. Данные обновляются раз в несколько минут,
// так что размер может не соответствовать действительности.
func (mc *MinioClient) GetBucketInfo() (*yaCloud.Bucket, error) {
	info, err := mc.yaCloud.GetBucket()

	if mc.err(err) {
		return nil, err
	}

	return info, nil
}

func (mc *MinioClient) GetBucketStats() (*yaCloud.BucketStats, error) {
	stats, err := mc.yaCloud.GetBucketStats()

	if mc.err(err) {
		return nil, err
	}

	return stats, err
}

func (mc *MinioClient) GetBucketInfoWithStats() (*yaCloud.BucketWithStats, error) {
	info, err := mc.GetBucketInfo()

	if mc.err(err) {
		return nil, err
	}

	stats, err := mc.GetBucketStats()

	if mc.err(err) {
		return nil, err
	}

	return &yaCloud.BucketWithStats{
		Bucket:      info,
		BucketStats: stats,
	}, nil
}

// EnableBucketVersioning - включает управление версиями файлов.
//
// Тонкости после включения:
//   - Для удаления пустой папки, которая была создана вручную, должен ставиться флаг permanently.
//   - Без флага permanently файлы не удаляются, им ставится новая нулевая версия с флагом IsDeleteMarker.
//     Чтобы увидеть все версии, в том числе с флагом IsDeleteMarker надо ставить флаг withVersions.
//     Чтобы удалить файл/папку вместе с версиями надо установить флаг permanently.
//   - Приостановка управления версиями автоматически не удаляет все неиспользуемые версии
func (mc *MinioClient) EnableBucketVersioning(ctx context.Context) error {
	err := mc.client.EnableVersioning(ctx, mc.BucketName)

	mc.err(err)
	return err
}

// SuspendBucketVersioning - приостанавливает управление версиями файлов.
//
// Приостановка управления версиями не удаляет неиспользуемые версии.
func (mc *MinioClient) SuspendBucketVersioning(ctx context.Context) error {
	err := mc.client.SuspendVersioning(ctx, mc.BucketName)

	mc.err(err)
	return err
}

// SetBucketQuota - устанавливает жесткое ограничение на размер хранилища.
//
// Таким образом, при попытке превысить квоту будет получена ошибка "Bucket quota exceeded".
func (mc *MinioClient) SetBucketQuota(quotaSize uint64) error {
	err := mc.yaCloud.UpdateBucketMaxSize(quotaSize)

	mc.err(err)
	return err
}

// GetBucketQuota - получить информацию об ограничении хранилища.
func (mc *MinioClient) GetBucketQuota() (string, error) {
	bucket, err := mc.yaCloud.GetBucket()
	if mc.err(err) {
		return "", err
	}

	return bucket.MaxSize, nil
}

// UnsetBucketQuota - снять ограничение на размер хранилища.
func (mc *MinioClient) UnsetBucketQuota() error {
	err := mc.yaCloud.UpdateBucketMaxSize(0)

	mc.err(err)
	return err
}
