package dal

import (
	"context"
	"gitea.pena/PenaSide/penadisk/dal/yaCloud"
	"github.com/minio/madmin-go/v2"
	"github.com/minio/minio-go/v7"
	"github.com/minio/minio-go/v7/pkg/credentials"
	"go.uber.org/zap"
	"time"
)

type MinioClient struct {
	client     *minio.Client
	yaCloud    *yaCloud.Client
	logger     *zap.Logger
	BucketName string
}

type MinioOptions struct {
	EndPoint        string
	AccessKeyID     string
	SecretAccessKey string
	UseSSL          bool
}

func NewMinioOptions(endPoint string, accessKeyID string, secretAccessKey string, useSSL bool) *MinioOptions {
	return &MinioOptions{EndPoint: endPoint, AccessKeyID: accessKeyID, SecretAccessKey: secretAccessKey, UseSSL: useSSL}
}

func NewClient(ctx context.Context, logger *zap.Logger, bucketName string, opts *MinioOptions) (*MinioClient, error) {
	// Create default client
	conn, err := minio.New(opts.EndPoint, &minio.Options{
		Creds:  credentials.NewStaticV4(opts.AccessKeyID, opts.SecretAccessKey, ""),
		Secure: opts.UseSSL,
	})
	if err != nil {
		logger.Error("MinioClient", zap.Error(err))
		return nil, err
	}

	return &MinioClient{
		client:     conn,
		yaCloud:    yaCloud.NewClient(bucketName),
		logger:     logger,
		BucketName: bucketName,
	}, nil
}

func ShowServerInfo(ctx context.Context, logger *zap.Logger, opts *MinioOptions) error {
	// Create admin client
	admin, err := madmin.New(opts.EndPoint, opts.AccessKeyID, opts.SecretAccessKey, opts.UseSSL)

	if err != nil {
		logger.Error("MinioClient", zap.Error(err))
		return err
	}

	// Show server info
	ctx, cancel := context.WithTimeout(ctx, 10*time.Second)
	defer cancel()
	serverInfo, err := admin.ServerInfo(ctx)

	if err != nil {
		logger.Error("MinioClient", zap.Error(err))
		return err
	}

	logger.Info("MinioClient", zap.Any("SERVER_INFO", serverInfo))

	return nil
}

func (mc *MinioClient) err(err error) bool {
	if err != nil {
		mc.logger.Error("MinioClient", zap.Error(err))
		return true
	}

	return false
}
