package yaCloud

import (
	"strconv"
	"time"
)

type ReqUpdateBucketMaxSize struct {
	UpdateMask string `json:"updateMask"`
	MaxSize    string `json:"maxSize"`
}

func NewReqUpdateBucketMaxSize(maxSize uint64) *ReqUpdateBucketMaxSize {
	return &ReqUpdateBucketMaxSize{
		UpdateMask: "maxSize",
		MaxSize:    strconv.FormatUint(maxSize, 10),
	}
}

type RespUpdateBucketMaxSize struct {
	ID          string `json:"id"`
	Description string `json:"description"`
	Done        bool   `json:"done"`
	CreatedBy   string `json:"createdBy"`
	Metadata    struct {
		Type string `json:"@type"`
		Name string `json:"name"`
	} `json:"metadata"`
	Response struct {
		Type string `json:"@type"`
		Bucket
	} `json:"response"`
	CreatedAt  time.Time `json:"createdAt"`
	ModifiedAt time.Time `json:"modifiedAt"`
}

type Bucket struct {
	Id                   string `json:"id"`
	Name                 string `json:"name"`
	FolderId             string `json:"folderId"`
	AnonymousAccessFlags struct {
		Read       bool `json:"read"`
		List       bool `json:"list"`
		ConfigRead bool `json:"configRead"`
	} `json:"anonymousAccessFlags"`
	DefaultStorageClass string `json:"defaultStorageClass"`
	Versioning          string `json:"versioning"`
	MaxSize             string `json:"maxSize"`
	Policy              any `json:"policy"`
	Acl                 struct {
		Grants []struct {
			Permission string `json:"permission"`
			GrantType  string `json:"grantType"`
			GranteeId  string `json:"granteeId"`
		} `json:"grants"`
	} `json:"acl"`
	CreatedAt time.Time `json:"createdAt"`
	Cors      []struct {
		Id             string   `json:"id"`
		AllowedMethods []string `json:"allowedMethods"`
		AllowedHeaders []string `json:"allowedHeaders"`
		AllowedOrigins []string `json:"allowedOrigins"`
		ExposeHeaders  []string `json:"exposeHeaders"`
		MaxAgeSeconds  string   `json:"maxAgeSeconds"`
	} `json:"cors"`
	WebsiteSettings struct {
		Index               string `json:"index"`
		Error               string `json:"error"`
		RedirectAllRequests struct {
			Protocol string `json:"protocol"`
			Hostname string `json:"hostname"`
		} `json:"redirectAllRequests"`
		RoutingRules []struct {
			Condition struct {
				HttpErrorCodeReturnedEquals string `json:"httpErrorCodeReturnedEquals"`
				KeyPrefixEquals             string `json:"keyPrefixEquals"`
			} `json:"condition"`
			Redirect struct {
				Hostname             string `json:"hostname"`
				HttpRedirectCode     string `json:"httpRedirectCode"`
				Protocol             string `json:"protocol"`
				ReplaceKeyPrefixWith string `json:"replaceKeyPrefixWith"`
				ReplaceKeyWith       string `json:"replaceKeyWith"`
			} `json:"redirect"`
		} `json:"routingRules"`
	} `json:"websiteSettings"`
	LifecycleRules []struct {
		Id      string `json:"id"`
		Enabled bool   `json:"enabled"`
		Filter  struct {
			Prefix                string `json:"prefix"`
			ObjectSizeGreaterThan string `json:"objectSizeGreaterThan"`
			ObjectSizeLessThan    string `json:"objectSizeLessThan"`
		} `json:"filter"`
		Expiration struct {
			Date                      string `json:"date"`
			Days                      string `json:"days"`
			ExpiredObjectDeleteMarker bool   `json:"expiredObjectDeleteMarker"`
		} `json:"expiration"`
		Transitions []struct {
			Date         string `json:"date"`
			Days         string `json:"days"`
			StorageClass string `json:"storageClass"`
		} `json:"transitions"`
		AbortIncompleteMultipartUpload struct {
			DaysAfterExpiration string `json:"daysAfterExpiration"`
		} `json:"abortIncompleteMultipartUpload"`
		NoncurrentExpiration struct {
			NoncurrentDays string `json:"noncurrentDays"`
		} `json:"noncurrentExpiration"`
		NoncurrentTransitions []struct {
			NoncurrentDays string `json:"noncurrentDays"`
			StorageClass   string `json:"storageClass"`
		} `json:"noncurrentTransitions"`
	} `json:"lifecycleRules"`
	Tags []struct {
		Key   string `json:"key"`
		Value string `json:"value"`
	} `json:"tags"`
	ObjectLock struct {
		Status           string `json:"status"`
		DefaultRetention struct {
			Mode  string `json:"mode"`
			Days  string `json:"days"`
			Years string `json:"years"`
		} `json:"defaultRetention"`
	} `json:"objectLock"`
}

type BucketStats struct {
	Name                 string `json:"name"`
	MaxSize              string `json:"maxSize"`
	UsedSize             string `json:"usedSize"`
	StorageClassMaxSizes []struct {
		StorageClass string `json:"storageClass"`
		ClassSize    string `json:"classSize"`
	} `json:"storageClassMaxSizes"`
	StorageClassUsedSizes []struct {
		StorageClass string `json:"storageClass"`
		ClassSize    string `json:"classSize"`
	} `json:"storageClassUsedSizes"`
	StorageClassCounters []struct {
		StorageClass string `json:"storageClass"`
		Counters     struct {
			SimpleObjectSize      string `json:"simpleObjectSize"`
			SimpleObjectCount     string `json:"simpleObjectCount"`
			ObjectsPartsSize      string `json:"objectsPartsSize"`
			ObjectsPartsCount     string `json:"objectsPartsCount"`
			MultipartObjectsSize  string `json:"multipartObjectsSize"`
			MultipartObjectsCount string `json:"multipartObjectsCount"`
			ActiveMultipartCount  string `json:"activeMultipartCount"`
		} `json:"counters"`
	} `json:"storageClassCounters"`
	DefaultStorageClass  string `json:"defaultStorageClass"`
	AnonymousAccessFlags struct {
		Read       bool `json:"read"`
		List       bool `json:"list"`
		ConfigRead bool `json:"configRead"`
	} `json:"anonymousAccessFlags"`
	CreatedAt time.Time `json:"createdAt"`
	UpdatedAt time.Time `json:"updatedAt"`
}

type BucketWithStats struct {
	*Bucket
	*BucketStats
}

type RespError struct {
	Code    int    `json:"code"`
	Message string `json:"message"`
	Details []struct {
		Type      string `json:"@type"`
		RequestId string `json:"requestId"`
	} `json:"details"`
}
